@@ -1,10 +1,15 @@
 #include <Foundation/Foundation.h>
+
 #include <mach-o/getsect.h>
 #include <mach-o/dyld.h>
-
+#include <mach/mach.h>
+#include <mach/mach_vm.h>
+#include <mach/vm_map.h>
+#include <mach/vm_page_size.h>
 #include <objc/message.h>
 #include <objc/runtime.h>
 
+#include <CoreGraphics/CoreGraphics.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
@@ -25,39 +30,62 @@
 #include "x86_64/payload.m"
 #elif __arm64__
 #include "arm64/payload.m"
+#include <ptrauth.h>
 #endif
 
+#define HASHTABLE_IMPLEMENTATION
+#include "../misc/hashtable.h"
+#undef HASHTABLE_IMPLEMENTATION
+
+#define page_align(addr) (vm_address_t)((uintptr_t)(addr) & (~(vm_page_size - 1)))
+
+#define unpack(b,v) memcpy(&v, b, sizeof(v)); b += sizeof(v)
+
+#define lerp(a, t, b) (((1.0-t)*a) + (t*b))
+
 #define SOCKET_PATH_FMT "/tmp/yabai-sa_%s.socket"
 
-#define BUF_SIZE 256
 #define kCGSOnAllWorkspacesTagBit (1 << 11)
 #define kCGSNoShadowTagBit (1 << 3)
 
 extern int CGSMainConnectionID(void);
 extern CGError CGSGetConnectionPSN(int cid, ProcessSerialNumber *psn);
+extern CGError CGSGetWindowAlpha(int cid, uint32_t wid, float *alpha);
 extern CGError CGSSetWindowAlpha(int cid, uint32_t wid, float alpha);
-extern CGError CGSSetWindowListAlpha(int cid, const uint32_t *window_list, int window_count, float alpha, float duration);
-extern CGError CGSSetWindowLevel(int cid, uint32_t wid, int level);
+extern CGError CGSSetWindowLevelForGroup(int cid, uint32_t wid, int level);
 extern OSStatus CGSMoveWindowWithGroup(const int cid, const uint32_t wid, CGPoint *point);
 extern CGError CGSReassociateWindowsSpacesByGeometry(int cid, CFArrayRef window_list);
 extern CGError CGSGetWindowOwner(int cid, uint32_t wid, int *window_cid);
-extern CGError CGSInvalidateWindowShadow(int cid, CGWindowID wid);
 extern CGError CGSSetWindowTags(int cid, uint32_t wid, const int tags[2], size_t maxTagSize);
 extern CGError CGSClearWindowTags(int cid, uint32_t wid, const int tags[2], size_t maxTagSize);
 extern CGError CGSGetWindowBounds(int cid, uint32_t wid, CGRect *frame);
 extern CGError CGSGetWindowTransform(int cid, uint32_t wid, CGAffineTransform *t);
 extern CGError CGSSetWindowTransform(int cid, uint32_t wid, CGAffineTransform t);
-extern CGError CGSAddWindowToWindowMovementGroup(int cid, uint32_t parent_wid, uint32_t child_wid);
-extern CGError CGSRemoveWindowFromWindowMovementGroup(int cid, uint32_t parent_wid, uint32_t child_wid);
-extern CGError CGSAddWindowToWindowOrderingGroup(int cid, uint32_t parent_wid, uint32_t child_wid, int order);
-extern CGError CGSRemoveFromOrderingGroup(int cid, uint32_t wid);
+extern CGError CGSOrderWindow(int cid, uint32_t wid, int order, uint32_t rel_wid);
 extern void CGSManagedDisplaySetCurrentSpace(int cid, CFStringRef display_ref, uint64_t spid);
 extern uint64_t CGSManagedDisplayGetCurrentSpace(int cid, CFStringRef display_ref);
-extern CFArrayRef CGSCopyManagedDisplaySpaces(const int cid);
 extern CFStringRef CGSCopyManagedDisplayForSpace(const int cid, uint64_t spid);
 extern void CGSShowSpaces(int cid, CFArrayRef spaces);
 extern void CGSHideSpaces(int cid, CFArrayRef spaces);
 
+extern CFTypeRef SLSTransactionCreate(int cid);
+extern CGError SLSTransactionCommit(CFTypeRef transaction, int unknown);
+extern CGError SLSTransactionOrderWindow(CFTypeRef transaction, uint32_t wid, int order, uint32_t rel_wid);
+extern CGError SLSTransactionSetWindowAlpha(CFTypeRef transaction, uint32_t wid, float alpha);
+extern CGError SLSTransactionSetWindowSystemAlpha(CFTypeRef transaction, uint32_t wid, float alpha);
+
+struct window_fade_context
+{
+    pthread_t thread;
+    uint32_t wid;
+    volatile float alpha;
+    volatile float duration;
+    volatile bool skip;
+};
+
+pthread_mutex_t window_fade_lock;
+struct table window_fade_table;
+
 static int _connection;
 static id dock_spaces;
 static id dp_desktop_picture_manager;
@@ -175,28 +203,59 @@ loop:
     return addr;
 }
 
+#if __arm64__
+uint64_t decode_adrp_add(uint64_t addr, uint64_t offset)
+{
+    uint32_t adrp_instr = *(uint32_t *) addr;
+
+    uint32_t immlo = (0x60000000 & adrp_instr) >> 29;
+    uint32_t immhi = (0xffffe0 & adrp_instr) >> 3;
+
+    int32_t value = (immhi | immlo) << 12;
+    int64_t value_64 = value;
+
+    uint32_t add_instr = *(uint32_t *) (addr + 4);
+    uint64_t imm12 = (add_instr & 0x3ffc00) >> 10;
+
+    if (add_instr & 0xc00000) {
+        imm12 <<= 12;
+    }
+
+    return (offset & 0xfffffffffffff000) + value_64 + imm12;
+}
+#endif
+
 static bool verify_os_version(NSOperatingSystemVersion os_version)
 {
     NSLog(@"[yabai-sa] checking for macOS %ld.%ld.%ld compatibility!", os_version.majorVersion, os_version.minorVersion, os_version.patchVersion);
 
-    if (os_version.majorVersion == 10) {
-        if (os_version.minorVersion == 13 && os_version.patchVersion == 6) {
-            return true; // High Sierra 10.13.6
-        } else if (os_version.minorVersion == 14 && os_version.patchVersion >= 4) {
-            return true; // Mojave 10.14.4-6
-        } else if (os_version.minorVersion == 15 && os_version.patchVersion >= 0) {
-            return true; // Catalina 10.15.0-6
-        } else if (os_version.minorVersion == 16) {
-            return true; // Big Sur 10.16 (Some beta versions)
-        }
-    } else if (os_version.majorVersion == 11) {
+#ifdef __x86_64__
+    if (os_version.majorVersion == 11) {
         return true; // Big Sur 11.0
+    } else if (os_version.majorVersion == 12) {
+        return true; // Monterey 12.0
+    } else if (os_version.majorVersion == 13) {
+        return true; // Ventura 13.0
     }
 
-    NSLog(@"[yabai-sa] spaces functionality is only supported on macOS High Sierra 10.13.6, Mojave 10.14.4-6, Catalina 10.15.0-6 and Big Sur 11.0");
+    NSLog(@"[yabai-sa] spaces functionality is only supported on macOS Big Sur 11.0-6, Monterey 12.0.0+, and Ventura 13.0.0");
+#elif __arm64__
+    if (os_version.majorVersion == 12) {
+        return true; // Monterey 12.0
+    } else if (os_version.majorVersion == 13) {
+        return true; // Ventura 13.0
+    }
+
+    NSLog(@"[yabai-sa] spaces functionality is only supported on macOS Monterey 12.0.0+, and Ventura 13.0.0");
+#endif
+
     return false;
 }
 
+#ifdef __x86_64__
+static double animation_time = 0.0001;
+#endif
+
 static void init_instances()
 {
     NSOperatingSystemVersion os_version = [[NSProcessInfo processInfo] operatingSystemVersion];
@@ -209,9 +268,15 @@ static void init_instances()
         dock_spaces = nil;
         NSLog(@"[yabai-sa] could not locate pointer to dock.spaces! spaces functionality will not work!");
     } else {
+#ifdef __x86_64__
         uint32_t dock_spaces_offset = *(int32_t *)dock_spaces_addr;
         NSLog(@"[yabai-sa] (0x%llx) dock.spaces found at address 0x%llX (0x%llx)", baseaddr, dock_spaces_addr, dock_spaces_addr - baseaddr);
         dock_spaces = [(*(id *)(dock_spaces_addr + dock_spaces_offset + 0x4)) retain];
+#elif __arm64__
+        uint64_t dock_spaces_offset = decode_adrp_add(dock_spaces_addr, dock_spaces_addr - baseaddr);
+        NSLog(@"[yabai-sa] (0x%llx) dock.spaces found at address 0x%llX (0x%llx)", baseaddr, dock_spaces_offset, dock_spaces_offset - baseaddr);
+        dock_spaces = [(*(id *)(baseaddr + dock_spaces_offset)) retain];
+#endif
     }
 
     uint64_t dppm_addr = hex_find_seq(baseaddr + get_dppm_offset(os_version), get_dppm_pattern(os_version));
@@ -219,9 +284,15 @@ static void init_instances()
         dp_desktop_picture_manager = nil;
         NSLog(@"[yabai-sa] could not locate pointer to dppm! moving spaces will not work!");
     } else {
+#ifdef __x86_64__
         uint32_t dppm_offset = *(int32_t *)dppm_addr;
         NSLog(@"[yabai-sa] (0x%llx) dppm found at address 0x%llX (0x%llx)", baseaddr, dppm_addr, dppm_addr - baseaddr);
         dp_desktop_picture_manager = [(*(id *)(dppm_addr + dppm_offset + 0x4)) retain];
+#elif __arm64__
+        uint64_t dppm_offset = decode_adrp_add(dppm_addr, dppm_addr - baseaddr);
+        NSLog(@"[yabai-sa] (0x%llx) dppm found at address 0x%llX (0x%llx)", baseaddr, dppm_offset, dppm_offset - baseaddr);
+        dp_desktop_picture_manager = [(*(id *)(baseaddr + dppm_offset)) retain];
+#endif
     }
 
     uint64_t add_space_addr = hex_find_seq(baseaddr + get_add_space_offset(os_version), get_add_space_pattern(os_version));
@@ -230,7 +301,11 @@ static void init_instances()
         add_space_fp = 0;
     } else {
         NSLog(@"[yabai-sa] (0x%llx) addSpace found at address 0x%llX (0x%llx)", baseaddr, add_space_addr, add_space_addr - baseaddr);
+#ifdef __x86_64__
         add_space_fp = add_space_addr;
+#elif __arm64__
+        add_space_fp = (uint64_t) ptrauth_sign_unauthenticated((void *) add_space_addr, ptrauth_key_asia, 0);
+#endif
     }
 
     uint64_t remove_space_addr = hex_find_seq(baseaddr + get_remove_space_offset(os_version), get_remove_space_pattern(os_version));
@@ -239,7 +314,11 @@ static void init_instances()
         remove_space_fp = 0;
     } else {
         NSLog(@"[yabai-sa] (0x%llx) removeSpace found at address 0x%llX (0x%llx)", baseaddr, remove_space_addr, remove_space_addr - baseaddr);
+#ifdef __x86_64__
         remove_space_fp = remove_space_addr;
+#elif __arm64__
+        remove_space_fp = (uint64_t) ptrauth_sign_unauthenticated((void *) remove_space_addr, ptrauth_key_asia, 0);
+#endif
     }
 
     uint64_t move_space_addr = hex_find_seq(baseaddr + get_move_space_offset(os_version), get_move_space_pattern(os_version));
@@ -248,7 +327,11 @@ static void init_instances()
         move_space_fp = 0;
     } else {
         NSLog(@"[yabai-sa] (0x%llx) moveSpace found at address 0x%llX (0x%llx)", baseaddr, move_space_addr, move_space_addr - baseaddr);
+#ifdef __x86_64__
         move_space_fp = move_space_addr;
+#elif __arm64__
+        move_space_fp = (uint64_t) ptrauth_sign_unauthenticated((void *) move_space_addr, ptrauth_key_asia, 0);
+#endif
     }
 
     uint64_t set_front_window_addr = hex_find_seq(baseaddr + get_set_front_window_offset(os_version), get_set_front_window_pattern(os_version));
@@ -257,89 +340,34 @@ static void init_instances()
         set_front_window_fp = 0;
     } else {
         NSLog(@"[yabai-sa] (0x%llx) setFrontWindow found at address 0x%llX (0x%llx)", baseaddr, set_front_window_addr, set_front_window_addr - baseaddr);
+#ifdef __x86_64__
         set_front_window_fp = set_front_window_addr;
+#elif __arm64__
+        set_front_window_fp = (uint64_t) ptrauth_sign_unauthenticated((void *) set_front_window_addr, ptrauth_key_asia, 0);
+#endif
+    }
+
+    uint64_t animation_time_addr = hex_find_seq(baseaddr + get_fix_animation_offset(os_version), get_fix_animation_pattern(os_version));
+    if (animation_time_addr == 0x0) {
+        NSLog(@"[yabai-sa] failed to get pointer to animation-time..");
+    } else {
+        NSLog(@"[yabai-sa] (0x%llx) animation_time_addr found at address 0x%llX (0x%llx)", baseaddr, animation_time_addr, animation_time_addr - baseaddr);
+        if (vm_protect(mach_task_self(), page_align(animation_time_addr), vm_page_size, 0, VM_PROT_READ | VM_PROT_WRITE | VM_PROT_COPY) == KERN_SUCCESS) {
+#ifdef __x86_64__
+            *(uint32_t *)(animation_time_addr + 4) = (uint32_t)((uint64_t)&animation_time - (animation_time_addr + 8));
+#elif __arm64__
+            *(uint32_t *) animation_time_addr = 0x2f00e400;
+#endif
+            vm_protect(mach_task_self(), page_align(animation_time_addr), vm_page_size, 0, VM_PROT_READ | VM_PROT_EXECUTE);
+        } else {
+            NSLog(@"[yabai-sa] animation_time_addr vm_protect failed; unable to patch instruction!");
+        }
     }
 
     managed_space = objc_getClass("Dock.ManagedSpace");
     _connection = CGSMainConnectionID();
 }
 
-typedef struct
-{
-    const char *text;
-    unsigned int length;
-} Token;
-
-static bool token_equals(Token token, const char *match)
-{
-    const char *at = match;
-    for (int i = 0; i < token.length; ++i, ++at) {
-        if ((*at == 0) || (token.text[i] != *at)) {
-            return false;
-        }
-    }
-    return *at == 0;
-}
-
-static uint64_t token_to_uint64t(Token token)
-{
-    uint64_t result = 0;
-    char buffer[token.length + 1];
-    memcpy(buffer, token.text, token.length);
-    buffer[token.length] = '\0';
-    sscanf(buffer, "%lld", &result);
-    return result;
-}
-
-static uint32_t token_to_uint32t(Token token)
-{
-    uint32_t result = 0;
-    char buffer[token.length + 1];
-    memcpy(buffer, token.text, token.length);
-    buffer[token.length] = '\0';
-    sscanf(buffer, "%d", &result);
-    return result;
-}
-
-static int token_to_int(Token token)
-{
-    int result = 0;
-    char buffer[token.length + 1];
-    memcpy(buffer, token.text, token.length);
-    buffer[token.length] = '\0';
-    sscanf(buffer, "%d", &result);
-    return result;
-}
-
-static float token_to_float(Token token)
-{
-    float result = 0.0f;
-    char buffer[token.length + 1];
-    memcpy(buffer, token.text, token.length);
-    buffer[token.length] = '\0';
-    sscanf(buffer, "%f", &result);
-    return result;
-}
-
-static Token get_token(const char **message)
-{
-    Token token;
-
-    token.text = *message;
-    while (**message && !isspace(**message)) {
-        ++(*message);
-    }
-    token.length = *message - token.text;
-
-    if (isspace(**message)) {
-        ++(*message);
-    } else {
-        // NOTE(koekeishiya): don't go past the null-terminator
-    }
-
-    return token;
-}
-
 static inline id get_ivar_value(id instance, const char *name)
 {
     id result = nil;
@@ -404,17 +432,16 @@ static inline id display_space_for_space_with_id(uint64_t space_id)
     return nil;
 }
 
-static void do_space_move(const char *message)
+static void do_space_move(char *message)
 {
-#ifdef __x86_64__
-    Token source_token = get_token(&message);
-    uint64_t source_space_id = token_to_uint64t(source_token);
+    if (dock_spaces == nil || dp_desktop_picture_manager == nil || move_space_fp == 0) return;
 
-    Token dest_token = get_token(&message);
-    uint64_t dest_space_id = token_to_uint64t(dest_token);
+    uint64_t source_space_id, dest_space_id;
+    unpack(message, source_space_id);
+    unpack(message, dest_space_id);
 
-    Token focus_token = get_token(&message);
-    bool focus_dest_space = token_to_int(focus_token);
+    bool focus_dest_space;
+    unpack(message, focus_dest_space);
 
     CFStringRef source_display_uuid = CGSCopyManagedDisplayForSpace(_connection, source_space_id);
     id source_space = space_for_display_with_id(source_display_uuid, source_space_id);
@@ -445,14 +472,16 @@ static void do_space_move(const char *message)
 
     CFRelease(source_display_uuid);
     CFRelease(dest_display_uuid);
-#endif
 }
 
 typedef void (*remove_space_call)(id space, id display_space, id dock_spaces, uint64_t space_id1, uint64_t space_id2);
-static void do_space_destroy(const char *message)
+static void do_space_destroy(char *message)
 {
-    Token space_id_token = get_token(&message);
-    uint64_t space_id = token_to_uint64t(space_id_token);
+    if (dock_spaces == nil || remove_space_fp == 0) return;
+
+    uint64_t space_id;
+    unpack(message, space_id);
+
     CFStringRef display_uuid = CGSCopyManagedDisplayForSpace(_connection, space_id);
     uint64_t active_space_id = CGSManagedDisplayGetCurrentSpace(_connection, display_uuid);
 
@@ -472,11 +501,13 @@ static void do_space_destroy(const char *message)
     CFRelease(display_uuid);
 }
 
-static void do_space_create(const char *message)
+static void do_space_create(char *message)
 {
-#ifdef __x86_64__
-    Token space_id_token = get_token(&message);
-    uint64_t space_id = token_to_uint64t(space_id_token);
+    if (dock_spaces == nil || add_space_fp == 0) return;
+
+    uint64_t space_id;
+    unpack(message, space_id);
+
     CFStringRef __block display_uuid = CGSCopyManagedDisplayForSpace(_connection, space_id);
     dispatch_sync(dispatch_get_main_queue(), ^{
         id new_space = [[managed_space alloc] init];
@@ -484,13 +515,15 @@ static void do_space_create(const char *message)
         asm__call_add_space(new_space, display_space, add_space_fp);
         CFRelease(display_uuid);
     });
-#endif
 }
 
-static void do_space_change(const char *message)
+static void do_space_focus(char *message)
 {
-    Token token = get_token(&message);
-    uint64_t dest_space_id = token_to_uint64t(token);
+    if (dock_spaces == nil) return;
+
+    uint64_t dest_space_id;
+    unpack(message, dest_space_id);
+
     if (dest_space_id) {
         CFStringRef dest_display = CGSCopyManagedDisplayForSpace(_connection, dest_space_id);
         id source_space = ((id (*)(id, SEL, CFStringRef)) objc_msgSend)(dock_spaces, @selector(currentSpaceforDisplayUUID:), dest_display);
@@ -512,14 +545,15 @@ static void do_space_change(const char *message)
                 }
             }
         }
+
         CFRelease(dest_display);
     }
 }
 
-static void do_window_scale(const char *message)
+static void do_window_scale(char *message)
 {
-    Token wid_token = get_token(&message);
-    uint32_t wid = token_to_uint32t(wid_token);
+    uint32_t wid;
+    unpack(message, wid);
     if (!wid) return;
 
     CGRect frame = {};
@@ -530,14 +564,11 @@ static void do_window_scale(const char *message)
     CGSGetWindowTransform(_connection, wid, &current_transform);
 
     if (CGAffineTransformEqualToTransform(current_transform, original_transform)) {
-        Token dx_token = get_token(&message);
-        float dx = token_to_float(dx_token);
-        Token dy_token = get_token(&message);
-        float dy = token_to_float(dy_token);
-        Token dw_token = get_token(&message);
-        float dw = token_to_float(dw_token);
-        Token dh_token = get_token(&message);
-        float dh = token_to_float(dh_token);
+        float dx, dy, dw, dh;
+        unpack(message, dx);
+        unpack(message, dy);
+        unpack(message, dw);
+        unpack(message, dh);
 
         int target_width  = dw / 4;
         int target_height = target_width / (frame.size.width/frame.size.height);
@@ -556,17 +587,15 @@ static void do_window_scale(const char *message)
     }
 }
 
-static void do_window_move(const char *message)
+static void do_window_move(char *message)
 {
-    Token wid_token = get_token(&message);
-    uint32_t wid = token_to_uint32t(wid_token);
+    uint32_t wid;
+    unpack(message, wid);
     if (!wid) return;
 
-    Token x_token = get_token(&message);
-    int x = token_to_int(x_token);
-
-    Token y_token = get_token(&message);
-    int y = token_to_int(y_token);
+    int x, y;
+    unpack(message, x);
+    unpack(message, y);
 
     CGPoint point = CGPointMake(x, y);
     CGSMoveWindowWithGroup(_connection, wid, &point);
@@ -576,49 +605,126 @@ static void do_window_move(const char *message)
     [window_list release];
 }
 
-static void do_window_alpha(const char *message)
+static void do_window_opacity(char *message)
 {
-    Token wid_token = get_token(&message);
-    uint32_t wid = token_to_uint32t(wid_token);
+    uint32_t wid;
+    unpack(message, wid);
     if (!wid) return;
 
-    Token alpha_token = get_token(&message);
-    float alpha = token_to_float(alpha_token);
-    CGSSetWindowAlpha(_connection, wid, alpha);
+    float alpha;
+    unpack(message, alpha);
+
+    pthread_mutex_lock(&window_fade_lock);
+    struct window_fade_context *context = table_find(&window_fade_table, &wid);
+
+    if (context) {
+        context->alpha = alpha;
+        context->duration = 0.0f;
+        __asm__ __volatile__ ("" ::: "memory");
+
+        context->skip = true;
+        pthread_mutex_unlock(&window_fade_lock);
+    } else {
+        CGSSetWindowAlpha(_connection, wid, alpha);
+        pthread_mutex_unlock(&window_fade_lock);
+    }
 }
 
-static void do_window_alpha_fade(const char *message)
+static void *window_fade_thread_proc(void *data)
 {
-    Token wid_token = get_token(&message);
-    uint32_t wid = token_to_uint32t(wid_token);
-    if (!wid) return;
+entry:;
+    struct window_fade_context *context = (struct window_fade_context *) data;
+    context->skip  = false;
 
-    Token alpha_token = get_token(&message);
-    float alpha = token_to_float(alpha_token);
-    Token duration_token = get_token(&message);
-    float duration = token_to_float(duration_token);
-    CGSSetWindowListAlpha(_connection, &wid, 1, alpha, duration);
+    float start_alpha;
+    float end_alpha = context->alpha;
+    CGSGetWindowAlpha(_connection, context->wid, &start_alpha);
+
+    int frame_duration = 8;
+    int total_duration = (int)(context->duration * 1000.0f);
+    int frame_count = (int)(((float) total_duration / (float) frame_duration) + 1.0f);
+
+    for (int frame_index = 1; frame_index <= frame_count; ++frame_index) {
+        if (context->skip) goto entry;
+
+        float t = (float) frame_index / (float) frame_count;
+        if (t < 0.0f) t = 0.0f;
+        if (t > 1.0f) t = 1.0f;
+
+        float alpha = lerp(start_alpha, t, end_alpha);
+        CGSSetWindowAlpha(_connection, context->wid, alpha);
+
+        usleep(frame_duration*1000);
+    }
+
+    pthread_mutex_lock(&window_fade_lock);
+    if (!context->skip) {
+        table_remove(&window_fade_table, &context->wid);
+        pthread_mutex_unlock(&window_fade_lock);
+        free(context);
+        return NULL;
+    }
+    pthread_mutex_unlock(&window_fade_lock);
+
+    goto entry;
 }
 
-static void do_window_level(const char *message)
+static void do_window_opacity_fade(char *message)
 {
-    Token wid_token = get_token(&message);
-    uint32_t wid = token_to_uint32t(wid_token);
+    uint32_t wid;
+    unpack(message, wid);
     if (!wid) return;
 
-    Token key_token = get_token(&message);
-    int key = token_to_int(key_token);
-    CGSSetWindowLevel(_connection, wid, CGWindowLevelForKey(key));
+    float alpha, duration;
+    unpack(message, alpha);
+    unpack(message, duration);
+
+    pthread_mutex_lock(&window_fade_lock);
+    struct window_fade_context *context = table_find(&window_fade_table, &wid);
+
+    if (context) {
+        context->alpha = alpha;
+        context->duration = duration;
+        __asm__ __volatile__ ("" ::: "memory");
+
+        context->skip = true;
+        pthread_mutex_unlock(&window_fade_lock);
+    } else {
+        context = malloc(sizeof(struct window_fade_context));
+        context->wid = wid;
+        context->alpha = alpha;
+        context->duration = duration;
+        context->skip = false;
+        __asm__ __volatile__ ("" ::: "memory");
+
+        table_add(&window_fade_table, &wid, context);
+        pthread_mutex_unlock(&window_fade_lock);
+        pthread_create(&context->thread, NULL, &window_fade_thread_proc, context);
+        pthread_detach(context->thread);
+    }
 }
 
-static void do_window_sticky(const char *message)
+static void do_window_layer(char *message)
 {
-    Token wid_token = get_token(&message);
-    uint32_t wid = token_to_uint32t(wid_token);
+    uint32_t wid;
+    unpack(message, wid);
     if (!wid) return;
 
-    Token value_token = get_token(&message);
-    int value = token_to_int(value_token);
+    int layer;
+    unpack(message, layer);
+
+    CGSSetWindowLevelForGroup(_connection, wid, CGWindowLevelForKey(layer));
+}
+
+static void do_window_sticky(char *message)
+{
+    uint32_t wid;
+    unpack(message, wid);
+    if (!wid) return;
+
+    bool value;
+    unpack(message, value);
+
     int tags[2] = { kCGSOnAllWorkspacesTagBit, 0 };
     if (value == 1) {
         CGSSetWindowTags(_connection, wid, tags, 32);
@@ -628,88 +734,76 @@ static void do_window_sticky(const char *message)
 }
 
 typedef void (*focus_window_call)(ProcessSerialNumber psn, uint32_t wid);
-static void do_window_focus(const char *message)
+static void do_window_focus(char *message)
 {
+    if (set_front_window_fp == 0) return;
+
     int window_connection;
     ProcessSerialNumber window_psn;
 
-    Token wid_token = get_token(&message);
-    uint32_t window_id = token_to_uint32t(wid_token);
+    uint32_t wid;
+    unpack(message, wid);
 
-    CGSGetWindowOwner(_connection, window_id, &window_connection);
+    CGSGetWindowOwner(_connection, wid, &window_connection);
     CGSGetConnectionPSN(window_connection, &window_psn);
 
-    ((focus_window_call) set_front_window_fp)(window_psn, window_id);
+    ((focus_window_call) set_front_window_fp)(window_psn, wid);
 }
 
-static void do_window_shadow(const char *message)
+static void do_window_shadow(char *message)
 {
-    Token wid_token = get_token(&message);
-    uint32_t wid = token_to_uint32t(wid_token);
+    uint32_t wid;
+    unpack(message, wid);
     if (!wid) return;
 
-    Token value_token = get_token(&message);
-    int value = token_to_int(value_token);
+    bool value;
+    unpack(message, value);
+
     int tags[2] = { kCGSNoShadowTagBit,  0};
     if (value == 1) {
         CGSClearWindowTags(_connection, wid, tags, 32);
     } else {
         CGSSetWindowTags(_connection, wid, tags, 32);
     }
-    CGSInvalidateWindowShadow(_connection, wid);
 }
 
-static void do_window_group_add(const char *message)
+static void do_window_swap_proxy(char *message)
 {
-    Token parent_token = get_token(&message);
-    uint32_t parent = token_to_uint32t(parent_token);
-    if (!parent) return;
+    uint32_t a_wid;
+    unpack(message, a_wid);
+    if (!a_wid) return;
 
-    Token child_token = get_token(&message);
-    uint32_t child = token_to_uint32t(child_token);
-    if (!child) return;
+    uint32_t b_wid;
+    unpack(message, b_wid);
+    if (!b_wid) return;
 
-    CGSAddWindowToWindowMovementGroup(_connection, parent, child);
-    CGSAddWindowToWindowOrderingGroup(_connection, parent, child, 1);
+    float alpha;
+    unpack(message, alpha);
+
+    int order;
+    unpack(message, order);
+
+    CFTypeRef transaction = SLSTransactionCreate(_connection);
+    SLSTransactionOrderWindow(transaction, b_wid, order, a_wid);
+    SLSTransactionSetWindowSystemAlpha(transaction, a_wid, alpha);
+    SLSTransactionCommit(transaction, 0);
+    CFRelease(transaction);
 }
 
-static void do_window_group_remove(const char *message)
+static void do_window_order(char *message)
 {
-    Token parent_token = get_token(&message);
-    uint32_t parent = token_to_uint32t(parent_token);
-    if (!parent) return;
+    uint32_t a_wid;
+    unpack(message, a_wid);
+    if (!a_wid) return;
 
-    Token child_token = get_token(&message);
-    uint32_t child = token_to_uint32t(child_token);
-    if (!child) return;
+    int order;
+    unpack(message, order);
 
-    CGSRemoveWindowFromWindowMovementGroup(_connection, parent, child);
-    CGSRemoveFromOrderingGroup(_connection, child);
-}
+    uint32_t b_wid;
+    unpack(message, b_wid);
+    if (!b_wid) return;
 
-static inline bool can_focus_space()
-{
-    return dock_spaces != nil;
-}
-
-static inline bool can_create_space()
-{
-    return dock_spaces != nil && add_space_fp != 0;
-}
-
-static inline bool can_destroy_space()
-{
-    return dock_spaces != nil && remove_space_fp != 0;
-}
-
-static inline bool can_move_space()
-{
-    return dock_spaces != nil && dp_desktop_picture_manager != nil && move_space_fp != 0;
-}
-
-static inline bool can_focus_window()
-{
-    return set_front_window_fp != 0;
+    CGSOrderWindow(_connection, a_wid, order, b_wid);
 }
 
 static void do_handshake(int sockfd)
@@ -736,71 +830,85 @@ static void do_handshake(int sockfd)
     send(sockfd, bytes, bytes_length+1, 0);
 }
 
-static void handle_message(int sockfd, const char *message)
+static void handle_message(int sockfd, char *message)
 {
-    /*
-     * NOTE(koekeishiya): interaction is supposed to happen through an
-     * external program (yabai), and so we do not bother doing input
-     * validation, as the program in question should do this.
-     */
-
-    Token token = get_token(&message);
-    if (token_equals(token, "handshake")) {
-        do_handshake(sockfd);
-    } else if (token_equals(token, "space")) {
-        if (!can_focus_space()) return;
-        do_space_change(message);
-    } else if (token_equals(token, "space_create")) {
-        if (!can_create_space()) return;
+    char op_code = *message++;
+    switch (op_code) {
+    case 0x01: {
+        do_space_focus(message);
+    } break;
+    case 0x02: {
         do_space_create(message);
-    } else if (token_equals(token, "space_destroy")) {
-        if (!can_destroy_space()) return;
+    } break;
+    case 0x03: {
         do_space_destroy(message);
-    } else if (token_equals(token, "space_move")) {
-        if (!can_move_space()) return;
+    } break;
+    case 0x04: {
         do_space_move(message);
-    } else if (token_equals(token, "window_scale")) {
-        do_window_scale(message);
-    } else if (token_equals(token, "window_move")) {
+    } break;
+    case 0x05: {
         do_window_move(message);
-    } else if (token_equals(token, "window_alpha")) {
-        do_window_alpha(message);
-    } else if (token_equals(token, "window_alpha_fade")) {
-        do_window_alpha_fade(message);
-    } else if (token_equals(token, "window_level")) {
-        do_window_level(message);
-    } else if (token_equals(token, "window_sticky")) {
+    } break;
+    case 0x06: {
+        do_window_opacity_fade(message);
+    } break;
+    case 0x07: {
+        do_window_layer(message);
+    } break;
+    case 0x08: {
         do_window_sticky(message);
-    } else if (token_equals(token, "window_focus")) {
-        if (!can_focus_window()) return;
-        do_window_focus(message);
-    } else if (token_equals(token, "window_shadow")) {
+    } break;
+    case 0x09: {
         do_window_shadow(message);
-    } else if (token_equals(token, "window_group_add")) {
-        do_window_group_add(message);
-    } else if (token_equals(token, "window_group_remove")) {
-        do_window_group_remove(message);
+    } break;
+    case 0x0A: {
+        do_window_focus(message);
+    } break;
+    case 0x0B: {
+        do_window_scale(message);
+    } break;
+    case 0x0C: {
+        do_handshake(sockfd);
+    } break;
+    case 0x0D: {
+        do_window_opacity(message);
+    } break;
+    case 0x0E: {
+        do_window_swap_proxy(message);
+    } break;
+    case 0x0F: {
+        do_window_order(message);
+    } break;
+
     }
 }
 
-static bool recv_socket(int sockfd, char *message, size_t message_size)
+static inline bool read_message(int sockfd, char *message)
 {
-    int len = recv(sockfd, message, message_size, 0);
-    if (len > 0) {
-        message[len] = '\0';
-        return true;
+    int bytes_read    = 0;
+    int bytes_to_read = 0;
+
+    if (read(sockfd, &bytes_to_read, sizeof(char)) == sizeof(char)) {
+        do {
+            int cur_read = read(sockfd, message+bytes_read, bytes_to_read-bytes_read);
+            if (cur_read <= 0) break;
+
+            bytes_read += cur_read;
+        } while (bytes_read < bytes_to_read);
+        return bytes_read == bytes_to_read;
     }
+
     return false;
 }
 
 static void *handle_connection(void *unused)
 {
-    while (1) {
+    for (;;) {
         int sockfd = accept(daemon_sockfd, NULL, 0);
         if (sockfd == -1) continue;
 
-        char message[BUF_SIZE];
-        if (recv_socket(sockfd, message, sizeof(message))) {
+        char message[0x100];
+        if (read_message(sockfd, message)) {
             handle_message(sockfd, message);
         }
 
@@ -811,6 +919,16 @@ static void *handle_connection(void *unused)
     return NULL;
 }
 
+static TABLE_HASH_FUNC(hash_wid)
+{
+    return *(uint32_t *) key;
+}
+
+static TABLE_COMPARE_FUNC(compare_wid)
+{
+    return *(uint32_t *) key_a == *(uint32_t *) key_b;
+}
+
 static bool start_daemon(char *socket_path)
 {
     struct sockaddr_un socket_address;
@@ -834,21 +952,20 @@ static bool start_daemon(char *socket_path)
         return false;
     }
 
+    init_instances();
+    pthread_mutex_init(&window_fade_lock, NULL);
+    table_init(&window_fade_table, 150, hash_wid, compare_wid);
     pthread_create(&daemon_thread, NULL, &handle_connection, NULL);
+
     return true;
 }
 
+__attribute__((constructor))
 void load_payload(void)
 {
-    NSLog(@"[yabai-sa] loaded payload");
-    init_instances();
+    NSLog(@"[yabai-sa] loaded payload..");
 
     const char *user = getenv("USER");
-    if (!user) {
-        NSString *ns_user = NSUserName();
-        if (ns_user) user = [ns_user UTF8String];
-    }
-
     if (!user) {
         NSLog(@"[yabai-sa] could not get 'env USER'! abort..");
         return;
@@ -863,14 +980,3 @@ void load_payload(void)
         NSLog(@"[yabai-sa] failed to spawn thread..");
     }
 }
-
-@interface Payload : NSObject
-+ (void) load;
-@end
-
-@implementation Payload
-+ (void) load
-{
-    load_payload();
-}
-@end
