@@ -254,7 +254,9 @@ static void window_node_split(struct view *view, struct window_node *node, struc
     struct window_node *right = malloc(sizeof(struct window_node));
     memset(right, 0, sizeof(struct window_node));
 
-    struct window_node *zoom = !node->zoom
+    struct window_node *zoom = !g_space_manager.window_zoom_persist
+                             ? NULL
+                             : !node->zoom
                              ? NULL
                              : node->zoom == node->parent
                              ? node
@@ -318,6 +320,16 @@ static void window_node_destroy(struct window_node *node)
     free(node);
 }
 
+static void window_node_clear_zoom(struct window_node *node)
+{
+    node->zoom = NULL;
+
+    if (!window_node_is_leaf(node)) {
+        window_node_clear_zoom(node->left);
+        window_node_clear_zoom(node->right);
+    }
+}
+
 void window_node_capture_windows(struct window_node *node, struct window_capture **window_list)
 {
     if (window_node_is_occupied(node)) {
@@ -651,7 +663,9 @@ struct window_node *view_remove_window_node(struct view *view, struct window *wi
 
     parent->left      = NULL;
     parent->right     = NULL;
-    parent->zoom      = !child->zoom
+    parent->zoom      = !g_space_manager.window_zoom_persist
+                      ? NULL
+                      : !child->zoom
                       ? NULL
                       : child->zoom == parent
                       ? parent->parent
@@ -668,7 +682,9 @@ struct window_node *view_remove_window_node(struct view *view, struct window *wi
     if (window_node_is_intermediate(child) && !window_node_is_leaf(child)) {
         parent->left          = child->left;
         parent->left->parent  = parent;
-        parent->left->zoom    = !child->left->zoom
+        parent->left->zoom    = !g_space_manager.window_zoom_persist
+                              ? NULL
+                              : !child->left->zoom
                               ? NULL
                               : child->left->zoom == child
                               ? parent
@@ -676,12 +692,18 @@ struct window_node *view_remove_window_node(struct view *view, struct window *wi
 
         parent->right         = child->right;
         parent->right->parent = parent;
-        parent->right->zoom   = !child->right->zoom
+        parent->right->zoom   = !g_space_manager.window_zoom_persist
+                              ? NULL
+                              : !child->right->zoom
                               ? NULL
                               : child->right->zoom == child
                               ? parent
                               : view->root;
 
+        if (!g_space_manager.window_zoom_persist) {
+            window_node_clear_zoom(parent);
+        }
+
         window_node_update(view, parent->left);
         window_node_update(view, parent->right);
     }
