@@ -560,7 +560,7 @@ void *window_manager_animate_window_list_thread_proc(void *data)
     struct window_animation_context *context = data;
     int animation_count = buf_len(context->animation_list);
 
-    ANIMATE(context->animation_connection, context->animation_duration, ease_out_cubic, {
+    ANIMATE(context->animation_connection, context->animation_frame_rate, context->animation_duration, ease_out_cubic, {
         for (int i = 0; i < animation_count; ++i) {
             if (context->animation_list[i].skip) continue;
 
@@ -600,6 +600,7 @@ void window_manager_animate_window_list_async(struct window_capture *window_list
 
     SLSNewConnection(0, &context->animation_connection);
     context->animation_duration = g_window_manager.window_animation_duration;
+    context->animation_frame_rate = g_window_manager.window_animation_frame_rate;
     context->animation_list = NULL;
 
     for (int i = 0; i < window_count; ++i) {
@@ -1201,6 +1202,15 @@ struct window *window_manager_find_second_cousin_for_managed_window(struct windo
 
 static void window_manager_make_key_window(ProcessSerialNumber *window_psn, uint32_t window_id)
 {
+    //
+    // :SynthesizedEvent
+    //
+    // NOTE(koekeishiya): These events will be picked up by an event-tap
+    // registered at the "Annotated Session" location; specifying that an
+    // event-tap is placed at the point where session events have been
+    // annotated to flow to an application.
+    //
+
     uint8_t bytes1[0xf8] = { [0x04] = 0xf8, [0x08] = 0x01, [0x3a] = 0x10 };
     uint8_t bytes2[0xf8] = { [0x04] = 0xf8, [0x08] = 0x02, [0x3a] = 0x10 };
 
@@ -1221,10 +1231,13 @@ void window_manager_focus_window_without_raise(ProcessSerialNumber *window_psn,
         memcpy(bytes1 + 0x3c, &g_window_manager.focused_window_id, sizeof(uint32_t));
         SLPSPostEventRecordTo(&g_window_manager.focused_window_psn, bytes1);
 
+        //
         // @hack
         // Artificially delay the activation by 1ms. This is necessary
         // because some applications appear to be confused if both of
         // the events appear instantaneously.
+        //
+
         usleep(10000);
 
         uint8_t bytes2[0xf8] = { [0x04] = 0xf8, [0x08] = 0x0d, [0x8a] = 0x01 };
@@ -1636,7 +1649,9 @@ enum window_op_error window_manager_warp_window(struct space_manager *sm, struct
 
     if (a_node == b_node) return WINDOW_OP_ERROR_SAME_STACK;
 
-    if (a_node->parent == b_node->parent && a_node->window_count == 1) {
+    if (a_node->parent && b_node->parent &&
+        a_node->parent == b_node->parent &&
+        a_node->window_count == 1) {
         if (window_node_contains_window(b_node, b_view->insertion_point)) {
             b_node->parent->split = b_node->split;
             b_node->parent->child = b_node->child;
@@ -2028,10 +2043,12 @@ void window_manager_toggle_window_parent(struct space_manager *sm, struct window
     struct window_node *node = view_find_window_node(view, window->id);
     assert(node);
 
+    if (!node->parent) return;
+
     if (node->zoom == node->parent) {
         node->zoom = NULL;
         window_node_flush(node);
-    } else if (node->parent) {
+    } else {
         node->zoom = node->parent;
         window_node_flush(node);
     }
@@ -2045,6 +2062,8 @@ void window_manager_toggle_window_fullscreen(struct space_manager *sm, struct wi
     struct window_node *node = view_find_window_node(view, window->id);
     assert(node);
 
+    if (node == view->root) return;
+
     if (node->zoom == view->root) {
         node->zoom = NULL;
         window_node_flush(node);
@@ -2268,6 +2287,7 @@ void window_manager_init(struct window_manager *wm)
     wm->normal_window_opacity = 1.0f;
     wm->window_opacity_duration = 0.0f;
     wm->window_animation_duration = 0.0f;
+    wm->window_animation_frame_rate = 120;
     wm->insert_feedback_windows = NULL;
     wm->insert_feedback_color = rgba_color_from_hex(0xffd75f5f);
     wm->active_border_color = rgba_color_from_hex(0xff775759);
